unit HelligdageU;

interface

uses
  SysUtils, Classes, Contnrs, Generics.Collections;
{$M+}
{$SCOPEDENUMS ON}

type
  TLanguage = (hdNative, hdEnglish);
  TCountryLocale = (Germany, Denmark, Norway, Sweden, USA, Greenland, France);

  THoliDay = class
  strict private
    FNativeName: string;
    FDate: TDateTime;
    FFriendlyDate: Boolean;
    FEnglishName: string;
    FLanguage: TLanguage;
    procedure SetDate(const Value: TDateTime);
    procedure SetNativeName(const Value: string);
    function GetDateAsStr: string;
    procedure SetFriendlyDate(const Value: Boolean);
    procedure SetEnglishName(const Value: string);
  public
    constructor Create(const Native, English: string; aDate: TDateTime);
    property NativeName: string read FNativeName write SetNativeName;
    property EnglishName: string read FEnglishName write SetEnglishName;
    property Date: TDateTime read FDate write SetDate;
    property DateAsStr: string read GetDateAsStr;
    property FriendlyDate: Boolean read FFriendlyDate write SetFriendlyDate;
  end;

  THolidayList = class(TObjectList<THoliDay>)
  public type
    TEnglishNameEnumerator = class(TEnumerator<string>)
    private
      FList: THolidayList;
      FIndex: Integer;
      function GetCurrent: string; virtual;
    protected
      function DoGetCurrent: string; override;
      function DoMoveNext: Boolean; override;
    public
      constructor Create(aList: THolidayList);
      property Current: string read GetCurrent;
      function MoveNext: Boolean;
    end;

    TNativeNameEnumerator = class(THolidayList.TEnglishNameEnumerator)
    private
      function GetCurrent: string; override;
    end;

    TEnglishNameCollection = class(TEnumerable<string>)
    private
      FList: THolidayList;
      function GetCount: Integer;
    protected
      function DoGetEnumerator: TEnumerator<string>; override;
    public
      constructor Create(ADictionary: THolidayList);
      function GetEnumerator: TEnglishNameEnumerator; reintroduce;
      function ToArray: TArray<string>; override; final;
      property Count: Integer read GetCount;
    end;

    TNativeNameCollection = class(THolidayList.TEnglishNameCollection)
    public
      function GetEnumerator: TNativeNameEnumerator; reintroduce;
    end;
  private
    FEnglishNames: TEnglishNameCollection;
    FNativeNames: TNativeNameCollection;

    function GetEnglishNames: TEnglishNameCollection;
    function GetNativeNames: TNativeNameCollection;
  public
    function Add(const Value: THoliDay): Integer; overload;
    function Add(const Native, English: string; aDate: TDateTime): Integer; overload;

    property NativeNames: TNativeNameCollection read GetNativeNames;
    property EnglishNames: TEnglishNameCollection read GetEnglishNames;
  end;

  TStdDato = class(THolidayList)
  private
    FYear: Integer;
    FCountry: TCountryLocale;
    FMarkSunday: Boolean;
    FMarkHolidays: Boolean;
    FLanguage: TLanguage;
    FSundayStr: string;
    function GeTHoliDay(Index: Integer): THoliDay;
    procedure SetYear(const Value: Integer);
    procedure SetCountry(const Value: TCountryLocale);
    procedure SetMarkSunday(const Value: Boolean);
    procedure SetMarkHolidays(const Value: Boolean);
    procedure SetLanguage(const Value: TLanguage);
  public
    constructor Create;
    function GetEasterDay(Year: Word = 0): TDateTime;
    procedure MakeHoliDays(Year: Word = 0);
    function GetHoliDaysInMonth(Month: Integer = 0; Year: Integer = 0): THolidayList;
    function GetHoliDayName(aDate: TDateTime): string;
    property HoliDays[Index: Integer]: THoliDay read GeTHoliDay;
  published
    property Country: TCountryLocale read FCountry write SetCountry;
    property Year: Integer read FYear write SetYear;
    property MarkSunday: Boolean read FMarkSunday write SetMarkSunday;
    property MarkHolidays: Boolean read FMarkHolidays write SetMarkHolidays;
    property Language: TLanguage read FLanguage write SetLanguage;
    property SundayStr: string read FSundayStr;
  end;

const
  CountryLocaleString: array [TCountryLocale] of string = ('Germany', 'Denmark', 'Norway', 'Sweden', 'USA', 'Greenland', 'France');

implementation

uses
  StrUtils, DateUtils;

type
  TNumber = (First = 1, Second = 2, Third = 3, Fourth = 4, Last = 5);
  TDay = (Monday = DayMonday, Tuesday = DayTuesday, Wednesday = DayWednesday, Thursday = DayThursday, Friday = DayFriday, Saturday = DaySaturday, Sunday = DaySunday);
  TMonth = (January = MonthJanuary, February = MonthFebruary, March = MonthMarch, April = MonthApril, May = MonthMay, June = MonthJune, July = MonthJuly, August = MonthAugust,
    September = MonthSeptember, October = MonthOctober, November = MonthNovember, December = MonthDecember);

function NthDayInMonth(const Number: TNumber; const Day: TDay; const Month: TMonth; const Year: Integer): TDateTime;
begin
  if Number = TNumber.Last then
  begin
    Result := IncDay(IncMonth(EncodeDate(Year, Integer(Month), 1), 1), -1);
    while DayOfTheWeek(Result) <> Integer(Day) do
      Result := IncDay(Result, -1);

    exit;
  end;

  Result := EncodeDate(Year, Integer(Month), 1);
  while DayOfTheWeek(Result) <> Integer(Day) do
    Result := IncDay(Result);

  Result := IncDay(Result, (Integer(Number) - 1) * DaysPerWeek);
end;

{ TStdDato }

constructor TStdDato.Create;
begin
  inherited;
  FCountry := TCountryLocale.Denmark;
  FYear := YearOf(now);
  FMarkSunday := True;
  FMarkHolidays := True;
  Language := TLanguage.hdNative;
end;

function TStdDato.GetEasterDay(Year: Word): TDateTime;
(*
  This algorithm is based in part on the algorithm of Oudin (1940) as quoted in
  "Explanatory Supplement to the Astronomical Almanac", P. Kenneth Seidelmann.

  G is the Golden Number-1
  H is 23-Epact (modulo 30)
  I is the number of days from 21 March to the Paschal full moon
  J is the weekday for the Paschal full moon (0=Sunday, 1=Monday, etc.)
  L is the number of days from 21 March to the Sunday on or before the Paschal full moon (a number between -6 and 28)
  C is then Century
*)
var
  G, H, I, J, L, C: Integer;
  X, Y, Z: Integer; // Just for temporary use
  EasterDay, EasterMonth: Integer;
begin
  if Year = 0 then
    Year := FYear;

  G := Year mod 19;
  C := Year div 100;

  X := C div 4;
  Y := (8 * C + 13) div 25;
  Z := 19 * G;
  H := (C - X - Y + Z + 15) mod 30;

  X := H div 28;
  Y := 29 div (H + 1);
  Z := (21 - G) div 11;
  I := H - X * (1 - Y * Z);

  X := Year div 4;
  Y := C div 4;
  J := (Year + X + I + 2 - C + Y) mod 7;

  L := I - J;
  X := (L + 40) div 44;
  EasterMonth := 3 + X;

  X := EasterMonth div 4;
  EasterDay := L + 28 - 31 * X;

  Result := EncodeDate(Year, EasterMonth, EasterDay);
end;

procedure TStdDato.MakeHoliDays(Year: Word);
var
  EDate, P: TDateTime;
  FirstSunday, LastSunday, aSunday: TDateTime;
begin
  if Year = 0 then
    Year := FYear;

  Clear;

  case Country of
    TCountryLocale.Denmark, TCountryLocale.Norway:
      FSundayStr := 'Sndag';
    TCountryLocale.Germany:
      FSundayStr := 'Sonntag';
    TCountryLocale.USA:
      FSundayStr := 'Sunday';
    TCountryLocale.Sweden:
      FSundayStr := 'Sndag';
    TCountryLocale.Greenland:
      FSundayStr := 'Sapaat';
    TCountryLocale.France:
      FSundayStr := 'Dimanche';
  end;

  if FMarkSunday then
  begin
    FirstSunday := NthDayInMonth(TNumber.First, TDay.Sunday, TMonth.January, Year);
    LastSunday := NthDayInMonth(TNumber.Last, TDay.Sunday, TMonth.January, Year);

    aSunday := FirstSunday;
    repeat
      Add(SundayStr, 'Sunday', aSunday);
      aSunday := IncDay(aSunday, DaysPerWeek);
    until aSunday > LastSunday;
  end;

  if not FMarkHolidays then
    exit;

  P := GetEasterDay(Year);

  case Country of
    TCountryLocale.Denmark:
      begin
        Add('Skrtorsdag', 'Maundy Thursday', IncDay(P, -3));
        Add('Langfredag', 'Good Friday', IncDay(P, -2));
        Add('Pskedag', 'Easter Sunday', P);
        Add('2. Pskedag', 'Easter Monday', IncDay(P, 1));
        Add('Store bededag', 'Prayer day (Local danish holiday)', IncDay(P, 26));
        Add('Kristi Himmelfartsdag', 'Ascension Day', IncDay(P, 39));
        Add('Pinsedag', 'White Sunday', IncDay(P, 49));
        Add('2. Pinsedag', 'White Monday', IncDay(P, 50));
        Add('Nytrsdag', 'New Year''s day', EncodeDate(Year, MonthJanuary, 1));
        Add('Grundlovsdag', 'Constitution Day', EncodeDate(Year, MonthJune, 5));
        Add('Juledag', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('2. Juledag', 'Boxing Day', EncodeDate(Year, MonthDecember, 26));
      end;
    TCountryLocale.Germany:
      begin
        Add('Karfreitag', 'Good Friday', IncDay(P, -2));
        Add('Ostersonntag', 'Easter Sunday', P);
        Add('Ostermontag', ' Easter Monday', IncDay(P, 1));
        Add('Christi Himmelfahrt', 'Ascension Day', IncDay(P, 39));
        Add('Pfingstsonntag', 'White Sunday', IncDay(P, 49));
        Add('Pfingstmontag', 'White Monday', IncDay(P, 50));

        // Gets 3rd Wednesday in November
        EDate := EncodeDate(Year, 11, 1);
        EDate := EDate + ((11 - DayOfWeek(EDate)) mod 7) + 14;
        Add('Bu- und Bettag', 'Day of prayer and repentance', EDate);
        Add('Neujahr', 'New Year''s day', EncodeDate(Year, MonthJanuary, 1));
        Add('Tag der Arbeit', 'Labor Day', EncodeDate(Year, MonthMay, 1));
        Add('Tag der deutschen Einheit', 'German reunification Day', EncodeDate(Year, MonthOctober, 3));
        Add('Reformationstag', 'Reformation Day', EncodeDate(Year, MonthOctober, 31));
        Add('Weihnachtsfeiertag', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('Weihnachtsfeiertag', 'Boxing Day', EncodeDate(Year, MonthDecember, 26));

        // Special thanx to Benny Baumann (BenBE@firemail.de)
        // For supplying me with the German holidays
      end;
    TCountryLocale.USA:
      begin
        Add('New Year''s Day', '', EncodeDate(Year, MonthJanuary, 1));

        // Third Monday in January.
        Add('Birthday of Martin Luther King', '', NthDayInMonth(TNumber.Third, TDay.Monday, TMonth.January, Year));

        // January 20th every four years, starting in 1937.
        if Year >= 1937 then
        begin
          if (Year - 1937) mod 4 = 0 then
            Add('Inauguration Day', '', EncodeDate(Year, MonthJanuary, 20)); // den nyvalgte USA-prsidents tiltrdelsesdag
        end
        else
        begin
          // March 4th every four years, pre-1937.
          if (1937 - Year) mod 4 = 0 then
            Add('Inauguration Day', '', EncodeDate(Year, MonthMarch, 4));
        end;

        // third Monday in February since 1971;
        // prior to that year, it was celebrated on the traditional date of February 22
        if Year >= 1971 then
          Add('Washington''s Birthday', '', NthDayInMonth(TNumber.Third, TDay.Monday, TMonth.February, Year))
        else
          Add('Washington''s Birthday', '', EncodeDate(Year, MonthFebruary, 22));

        // Third Saturday in May.
        Add('Armed Forces Day', '', NthDayInMonth(TNumber.Third, TDay.Saturday, TMonth.May, Year));

        // from 1868 to 1970 it was celebrated on May 30, and was called Decoration Day for part of that time.
        if (Year >= 1868) and (Year <= 1970) then
          Add('Decoration Day', '', EncodeDate(Year, MonthMay, 30))
        else if Year >= 1971 then // last Monday in May since 1971
          Add('Memorial Day', '', NthDayInMonth(TNumber.Last, TDay.Monday, TMonth.May, Year));

        Add('Flag Day', '', EncodeDate(Year, MonthJune, 14));
        Add('United States of America''s Independence Day', '', EncodeDate(Year, MonthJuly, 4));

        // first Monday in September
        Add('Labor Day', '', NthDayInMonth(TNumber.First, TDay.Monday, TMonth.September, Year));

        // second Monday in October (federal holiday since 1971).
        if Year >= 1971 then
          Add('Columbus Day', '', NthDayInMonth(TNumber.Second, TDay.Monday, TMonth.October, Year));

        // Tuesday on or after November 2. every 4. year since 1820
        if (Year >= 1820) and (Year mod 4 = 0) then
        begin
          EDate := EncodeDate(Year, MonthNovember, 2);
          while DayOfTheWeek(EDate) <> DayTuesday do
            EDate := IncDay(EDate);

          Add('Election Day', '', EDate);
        end;

        // November 11th
        // except from 1971 to 1977, inclusive,
        // when it was celebrated on the fourth Monday in October;
        // formerly known as Armistice.
        if Year > 1978 then
          Add('Veterans Day', '', EncodeDate(Year, MonthNovember, 11))
        else if Year >= 1971 then
          Add('Armistice', '', NthDayInMonth(TNumber.Fourth, TDay.Monday, TMonth.November, Year));

        // fourth Thursday in November.
        Add('Thanksgiving Day', '', NthDayInMonth(TNumber.Fourth, TDay.Tuesday, TMonth.November, Year));
        Add('Christmas Day', '', EncodeDate(Year, MonthDecember, 25));
        Add('Good Friday', '', IncDay(P, -2));
        Add('Easter Sunday', '', P);

        // Source : www.smart.net/~mmontes/ushols.html
        // For a further explanation of each Holiday :
        // http://www.bright.net/~jimsjems/presidnt.html
      end; // USA
    TCountryLocale.Sweden:
      begin
        Add('Lngfredag', 'Good Friday', IncDay(P, -2));
        Add('Pskdagen', 'Easter Sunday', P);
        Add('Annandag psk', 'Easter Monday', IncDay(P, 1));
        Add('Kristi himmelsfrdsdag', 'Ascension Day', IncDay(P, 39));
        Add('Pingstdagen', 'White Sunday', IncDay(P, 49));
        Add('Annandag Pingst', 'White Monday', IncDay(P, 50));

        // First sunday after June 19.

        EDate := EncodeDate(Year, MonthJune, 19);
        while DayOfTheWeek(EDate) <> DaySunday do
          EDate := IncDay(EDate);

        Add('Midsommardagen', 'Midsummer Day', EDate);

        EDate := EncodeDate(Year, MonthOctober, 30);
        while DayOfTheWeek(EDate) <> DaySaturday do
          EDate := IncDay(EDate);

        // Saturday after 30 October
        Add('Alla helgons dag', 'All Saints'' Eve', EDate);
        Add('Nyrsdagen', 'New Year''s day', EncodeDate(Year, MonthJanuary, 1));
        Add('Trettondagen', 'Epiphany', EncodeDate(Year, MonthJanuary, 6));
        Add('Frsta maj', 'Labor Day', EncodeDate(Year, MonthMay, 1));
        Add('Juldagen', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('Annandag jul', 'Boxing Day', EncodeDate(Year, MonthDecember, 26));

        // source : http://www.cmb.gu.se/intbla11.htm
        // Speciial thanx to Dan Nordin (Dan@techotel.se)
        // For supplying me with the swedish names
      end; // Sweden
    TCountryLocale.Norway:
      begin
        Add('Palmesndag', 'Palm Sunday', IncDay(P, -DaysPerWeek));
        Add('Skjrtorsdag', 'Maundy Thursday', IncDay(P, -3));
        Add('Langfredag', 'Good Friday', IncDay(P, -2));
        Add('1. Pskedag ', 'Easter Sunday', P);
        Add('2. Pskedag ', 'Easter Monday', IncDay(P, 1));
        Add('Kristi Himmelfart', 'Ascension Day', IncDay(P, 39));
        Add('1. Pinsedag ', 'White Sunday', IncDay(P, 49));
        Add('2. Pinsedag ', 'White Monday', IncDay(P, 50));
        Add('1. Nyttrsdag ', 'New Year''s Day', EncodeDate(Year, MonthJanuary, 1));
        Add('Arbeidernes dag ', 'Labor Day', EncodeDate(Year, MonthMay, 1));
        Add('Nasjonaldag', 'Constitution Day', EncodeDate(Year, MonthMay, 17));
        Add('1. Juledag ', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('2. Juledag', 'Boxing Day', EncodeDate(Year, MonthDecember, 26));

        // source : http://www.ssb.no/english/subjects/00/minifakta_en/en/02.html
        // Speciial thanx to AUD BERGQUIST (sollemor@c2i.net)
        // For supplying me with the norwegien names
      end; // Norway

    TCountryLocale.Greenland:
      begin
        Add('Skrtorsdag', 'Maundy Thursday', IncDay(P, -3));
        Add('Langfredag', 'Good Friday', IncDay(P, -2));
        Add('Pskedag', 'Easter Sunday', P);
        Add('2. Pskedag', 'Easter Monday', IncDay(P, 1));
        Add('Store bededag', 'Prayer day (Local danish holiday)', IncDay(P, 26));
        Add('Kristi Himmelfartsdag', 'Ascension Day', IncDay(P, 39));
        Add('Pinsedag', 'Whit Sunday', IncDay(P, 49));
        Add('2. Pinsedag', 'Whit Monday', IncDay(P, 50));
        Add('Nytrsdag', 'New Year''s day', EncodeDate(Year, MonthJanuary, 1));
        Add('Grundlovsdag', 'Constitution Day', EncodeDate(Year, MonthJune, 5));
        Add('Inuiattut ullorsiorneq', 'Constitution Day', EncodeDate(Year, MonthJune, 21));
        Add('Juledag', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('2. Juledag', 'Boxing Day', EncodeDate(Year, MonthDecember, 26));

        // Source : http://www.holidayfestival.com/Greenland.html
        // Special thanx to nanoq aka Mikael Pedersen (nanoq@nanoq.dk)
        // For suplying me with the Greenlandish names.
      end;

    TCountryLocale.France:
      begin
        Add('Nouvel an', 'New Year''s Day', EncodeDate(Year, MonthJanuary, 1));
        Add('Fte du travail', 'Labor Day', EncodeDate(Year, MonthMay, 1));
        Add('Fte de la Victoire 1945', 'WWII Victory Day', EncodeDate(Year, MonthMay, 8));
        Add('Fte nationale', 'Bastille Day', EncodeDate(Year, MonthJuly, 14));
        Add('Assumption', 'Assumption', EncodeDate(Year, MonthAugust, 15));
        Add('Toussaint', 'All Saints Day', EncodeDate(Year, MonthNovember, 1));
        Add('Armistice', 'Armistice Day', EncodeDate(Year, MonthNovember, 11));
        Add('Noel', 'Christmas Day', EncodeDate(Year, MonthDecember, 25));
        Add('Paques', 'Easter Sunday', P);
        Add('Lundi de paques', 'Easter Monday', IncDay(P, 1));
        Add('Ascension', 'Ascension Day', IncDay(P, 39));
        Add('Pentecte', 'Whit Sunday', IncDay(P, 49));
        Add('Lundi de pentecte', 'Whit Monday', IncDay(P, 50));
      end; // France
  end;
end;

function TStdDato.GeTHoliDay(Index: Integer): THoliDay;
begin
  Result := nil;
  if Index < Count then
    Result := Items[Index];
end;

function TStdDato.GetHoliDaysInMonth(Month, Year: Integer): THolidayList;
var
  aYear, aMonth, aDay: Word;
  HoliDay: THoliDay;
begin
  Result := THolidayList.Create(false);

  if Year = 0 then
    Year := FYear;

  if Month = 0 then
    Year := FYear;

  for HoliDay in Self do
  begin
    DecodeDate(HoliDay.Date, aYear, aMonth, aDay);
    if (aMonth = Month) and (aYear = Year) then
      Result.Add(HoliDay)
  end;
end;

procedure TStdDato.SetYear(const Value: Integer);
begin
  FYear := Value;
  MakeHoliDays(0);
end;

function IfThen(AValue: Boolean; const ATrue: string; AFalse: string = ''): string;
begin
  if AValue then
    Result := ATrue
  else
    Result := AFalse;
end;

function TStdDato.GetHoliDayName(aDate: TDateTime): string;
var
  HoliDay: THoliDay;
begin
  Result := '';
  for HoliDay in Self do
    if HoliDay.Date = Trunc(aDate) then
      exit(IfThen(FLanguage = TLanguage.hdNative, HoliDay.NativeName, HoliDay.EnglishName));
end;

procedure TStdDato.SetCountry(const Value: TCountryLocale);
begin
  FCountry := Value;
end;

procedure TStdDato.SetMarkSunday(const Value: Boolean);
begin
  FMarkSunday := Value;
end;

procedure TStdDato.SetMarkHolidays(const Value: Boolean);
begin
  FMarkHolidays := Value;
end;

procedure TStdDato.SetLanguage(const Value: TLanguage);
begin
  FLanguage := Value;
end;

{ THoliDay }

constructor THoliDay.Create(const Native, English: string; aDate: TDateTime);
begin
  FDate := aDate;
  FNativeName := Native;
  FEnglishName := IfThen(English = '', Native, English);
  FLanguage := TLanguage.hdNative;
  FFriendlyDate := True;
end;

function THoliDay.GetDateAsStr: string;
begin
  if not FFriendlyDate then
    Result := DateToStr(FDate)
  else
    Result := FormatDateTime(FormatSettings.LongDateFormat, FDate);
end;

procedure THoliDay.SetDate(const Value: TDateTime);
begin
  FDate := Value;
end;

procedure THoliDay.SetEnglishName(const Value: string);
begin
  FEnglishName := Value;
end;

procedure THoliDay.SetFriendlyDate(const Value: Boolean);
begin
  FFriendlyDate := Value;
end;

procedure THoliDay.SetNativeName(const Value: string);
begin
  FNativeName := Value;
end;

{ THolidayList.TEnglishNameEnumerator }

constructor THolidayList.TEnglishNameEnumerator.Create(aList: THolidayList);
begin
  inherited Create;
  FIndex := -1;
  FList := aList;
end;

function THolidayList.TEnglishNameEnumerator.DoGetCurrent: string;
begin
  Result := GetCurrent;
end;

function THolidayList.TEnglishNameEnumerator.DoMoveNext: Boolean;
begin
  Result := MoveNext;
end;

function THolidayList.TEnglishNameEnumerator.GetCurrent: string;
begin
  Result := FList[FIndex].EnglishName;
end;

function THolidayList.TEnglishNameEnumerator.MoveNext: Boolean;
begin
  if FIndex >= FList.Count then
    exit(false);
  Inc(FIndex);
  Result := FIndex < FList.Count;
end;

{ THolidayList.TNativeNameEnumerator }

function THolidayList.TNativeNameEnumerator.GetCurrent: string;
begin
  Result := FList[FIndex].NativeName;
end;

{ THolidayList }

function THolidayList.Add(const Native, English: string; aDate: TDateTime): Integer;
begin
  exit(Add(THoliDay.Create(Native, English, aDate)));
end;

function THolidayList.Add(const Value: THoliDay): Integer;
begin
  exit( inherited Add(Value));
end;

function THolidayList.GetEnglishNames: TEnglishNameCollection;
begin
  if FEnglishNames = nil then
    FEnglishNames := TEnglishNameCollection.Create(Self);
  Result := FEnglishNames;
end;

function THolidayList.GetNativeNames: TNativeNameCollection;
begin
  if FNativeNames = nil then
    FNativeNames := TNativeNameCollection.Create(Self);
  Result := FNativeNames;
end;

{ THolidayList.TEnglishNameCollection }

constructor THolidayList.TEnglishNameCollection.Create(ADictionary: THolidayList);
begin
  inherited Create;
  FList := ADictionary;
end;

function THolidayList.TEnglishNameCollection.DoGetEnumerator: TEnumerator<string>;
begin
  Result := GetEnumerator;
end;

function THolidayList.TEnglishNameCollection.GetCount: Integer;
begin
  Result := FList.Count;
end;

function THolidayList.TEnglishNameCollection.GetEnumerator: TEnglishNameEnumerator;
begin
  Result := TEnglishNameEnumerator.Create(FList);
end;

function THolidayList.TEnglishNameCollection.ToArray: TArray<string>;
begin
  Result := ToArrayImpl(FList.Count);
end;

{ THolidayList.TNativeNameCollection }

function THolidayList.TNativeNameCollection.GetEnumerator: TNativeNameEnumerator;
begin
  Result := TNativeNameEnumerator.Create(FList);
end;

end.
